defmodule Nulla.Notes.Note do
  use Ecto.Schema
  import Ecto.Changeset
  alias Nulla.Snowflake
  alias Nulla.Actors.Actor
  alias Nulla.MediaAttachments.MediaAttachment

  @primary_key {:id, :integer, autogenerate: false}
  schema "notes" do
    field :inReplyTo, :string
    field :published, :utc_datetime
    field :url, :string
    field :visibility, :string
    field :to, {:array, :string}
    field :cc, {:array, :string}
    field :sensitive, :boolean, default: false
    field :content, :string
    field :language, :string

    belongs_to :actor, Actor
    has_many :media_attachments, MediaAttachment

    timestamps(type: :utc_datetime)
  end

  @doc false
  def changeset(note, attrs) do
    note
    |> cast(attrs, [
      :inReplyTo,
      :published,
      :url,
      :visibility,
      :to,
      :cc,
      :sensitive,
      :content,
      :language,
      :actor_id
    ])
    |> maybe_put_id()
    |> validate_required([
      :published,
      :url,
      :visibility,
      :to,
      :cc,
      :content,
      :language,
      :actor_id
    ])
  end

  defp maybe_put_id(changeset) do
    id_in_attrs = get_field(changeset, :id)

    if is_nil(id_in_attrs) do
      change(changeset, id: Snowflake.next_id())
    else
      changeset
    end
  end
end
