defmodule Nulla.Types.StringOrJson do
  @behaviour Ecto.Type

  @impl true
  def type, do: :string

  @impl true
  def cast(value) when is_map(value) or is_list(value), do: {:ok, value}

  @impl true
  def cast(value) when is_binary(value) do
    case Jason.decode(value) do
      {:ok, decoded} -> {:ok, decoded}
      _ -> {:ok, value}
    end
  end

  @impl true
  def cast(_), do: :error

  @impl true
  def dump(value) when is_map(value) or is_list(value), do: Jason.encode(value)

  @impl true
  def dump(value) when is_binary(value), do: {:ok, value}

  @impl true
  def dump(_), do: :error

  @impl true
  def load(value) when is_binary(value) do
    case Jason.decode(value) do
      {:ok, decoded} -> {:ok, decoded}
      _ -> {:ok, value}
    end
  end

  @impl true
  def load(_), do: :error

  @impl true
  def embed_as(_format), do: :self

  @impl true
  def equal?(term1, term2), do: term1 == term2
end
